@@ -1,29 +1,62 @@
-use core::mem;
-use core::ptr;
-use core::ptr::NonNull;
-use core::slice;
-use core::iter::FusedIterator;
+use crate::*;
 
-pub struct Drain<'a, T: 'a, const N: usize> {
+pub(crate) fn slice_range<R>(range: R, bounds: ops::RangeTo<usize>) -> ops::Range<usize>
+where
+    R: ops::RangeBounds<usize>,
+{
+    let len = bounds.end;
+
+    let start: ops::Bound<&usize> = range.start_bound();
+    let start = match start {
+        ops::Bound::Included(&start) => start,
+        ops::Bound::Excluded(start) => start
+            .checked_add(1)
+            .unwrap_or_else(|| panic!("attempted to index slice from after maximum usize")),
+        ops::Bound::Unbounded => 0,
+    };
+
+    let end: ops::Bound<&usize> = range.end_bound();
+    let end = match end {
+        ops::Bound::Included(end) => end
+            .checked_add(1)
+            .unwrap_or_else(|| panic!("attempted to index slice up to maximum usize")),
+        ops::Bound::Excluded(&end) => end,
+        ops::Bound::Unbounded => len,
+    };
+
+    if start > end {
+        panic!("slice index starts at {start} but ends at {end}");
+    }
+    if end > len {
+        panic!("range end index {end} out of range for slice of length {len}",);
+    }
+    ops::Range { start, end }
+}
+
+pub struct Drain<'a, T: 'a, A: Array<T>> {
     /// Index of tail to preserve
     pub(super) tail_start: usize,
     /// Length of tail
     pub(super) tail_len: usize,
     /// Current remaining range to remove
     pub(super) iter: slice::Iter<'a, T>,
-    pub(super) arr: NonNull<super::Array<T, N>>,
+    pub(super) vec: NonNull<A>,
 }
 
-impl<'a, T, const N: usize> Drain<'a, T, N> {
+impl<T: fmt::Debug, A: Array<T>> fmt::Debug for Drain<'_, T, A> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_tuple("Drain").field(&self.iter.as_slice()).finish()
+    }
+}
+
+impl<'a, T, A: Array<T>> Drain<'a, T, A> {
     /// Returns the remaining items of this iterator as a slice.
     ///
     /// # Examples
     ///
     /// ```
-    /// use stack_array::Array;
-    ///
-    /// let mut arr: Array<char, 3> = Array::from(['a', 'b', 'c']);
-    /// let mut drain = arr.drain(..);
+    /// let mut vec = vec!['a', 'b', 'c'];
+    /// let mut drain = vec.drain(..);
     /// assert_eq!(drain.as_slice(), &['a', 'b', 'c']);
     /// let _ = drain.next().unwrap();
     /// assert_eq!(drain.as_slice(), &['b', 'c']);
@@ -34,10 +67,16 @@ impl<'a, T, const N: usize> Drain<'a, T, N> {
     }
 }
 
-unsafe impl<T: Sync, const N: usize> Sync for Drain<'_, T, N> {}
-unsafe impl<T: Send, const N: usize> Send for Drain<'_, T, N> {}
+impl<'a, T, A: Array<T>> AsRef<[T]> for Drain<'a, T, A> {
+    fn as_ref(&self) -> &[T] {
+        self.as_slice()
+    }
+}
 
-impl<T, const N: usize> Iterator for Drain<'_, T, N> {
+unsafe impl<T: Sync, A: Sync + Array<T>> Sync for Drain<'_, T, A> {}
+unsafe impl<T: Send, A: Send + Array<T>> Send for Drain<'_, T, A> {}
+
+impl<T, A: Array<T>> Iterator for Drain<'_, T, A> {
     type Item = T;
 
     #[inline]
@@ -52,7 +91,7 @@ impl<T, const N: usize> Iterator for Drain<'_, T, N> {
     }
 }
 
-impl<T, const N: usize> DoubleEndedIterator for Drain<'_, T, N> {
+impl<T, A: Array<T>> DoubleEndedIterator for Drain<'_, T, A> {
     #[inline]
     fn next_back(&mut self) -> Option<T> {
         self.iter
@@ -61,16 +100,16 @@ impl<T, const N: usize> DoubleEndedIterator for Drain<'_, T, N> {
     }
 }
 
-impl<T, const N: usize> Drop for Drain<'_, T, N> {
+impl<T, A: Array<T>> Drop for Drain<'_, T, A> {
     fn drop(&mut self) {
         /// Moves back the un-`Drain`ed elements to restore the original `Vec`.
-        struct DropGuard<'r, 'a, T, const N: usize>(&'r mut Drain<'a, T, N>);
+        struct DropGuard<'r, 'a, T, A: Array<T>>(&'r mut Drain<'a, T, A>);
 
-        impl<'r, 'a, T, const N: usize> Drop for DropGuard<'r, 'a, T, N> {
+        impl<'r, 'a, T, A: Array<T>> Drop for DropGuard<'r, 'a, T, A> {
             fn drop(&mut self) {
                 if self.0.tail_len > 0 {
                     unsafe {
-                        let source_vec = self.0.arr.as_mut();
+                        let source_vec = self.0.vec.as_mut();
                         // memmove back untouched tail, update to new length
                         let start = source_vec.len();
                         let tail = self.0.tail_start;
@@ -88,7 +127,7 @@ impl<T, const N: usize> Drop for Drain<'_, T, N> {
         let iter = mem::replace(&mut self.iter, (&mut []).iter());
         let drop_len = iter.len();
 
-        let mut vec = self.arr;
+        let mut vec = self.vec;
 
         if mem::size_of::<T>() == 0 {
             // ZSTs have no identity, so we don't need to move them around, we only need to drop the correct amount.
@@ -129,5 +168,4 @@ impl<T, const N: usize> Drop for Drain<'_, T, N> {
     }
 }
 
-impl<T, const N: usize> ExactSizeIterator for Drain<'_, T, N> {}
-impl<T, const N: usize> FusedIterator for Drain<'_, T, N> {}
+impl<T, A: Array<T>> core::iter::FusedIterator for Drain<'_, T, A> {}
